%
%    Copyright (C) 2008  Colin Caprani - www.colincaprani.com
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

function out = ppHAHBrun(output,step,varargin)
% From a matrix of analysis results, this function finds the maximum and
% minimum load effects, and the positions at which they occur. It returns
% this information as well as outputting it to the console and plotting the
% bending moment and shear force diagrams for the maximum and minimum cases
% as well as the envelopes of moment and shear at each section

[nrows ncols] = size(output);
output(2:nrows,2:ncols) = round(output(2:nrows,2:ncols)*100)/100;
x = output(2:nrows,1);
for i = 2:ncols
    mts = output(2:nrows,i);
    maxMt = max(mts); [imax j] = find(mts == maxMt);
    minMt = min(mts); [imin j] = find(mts == minMt);
    % the min just picks the first place of the value
    % in case of multiple locations
    maxPos = x(min(imax));  minPos = x(min(imin));
    out(i-1,:) = [output(1,i), maxMt, maxPos, minMt, minPos];
end

disp('----- RESULTS -----');
disp(sprintf('Load At \tMax Mt  \tAt  \tMin Mt  \tAt'));
maxMt = max(out(:,2)); minMt = min(out(:,4)); % these may be vectors
[imax j] = find(out == maxMt); disp('Max:'); %disp(out(imax,:))
for i = 1:length(imax)
    disp(sprintf('%3.3f\t\t%0.5g\t\t%3.3f\t%0.5g\t%3.3f',out(imax(i),:))); 
end
[imin j] = find(out == minMt); disp('Min:'); %disp(out(imin,:))
for i = 1:length(imin)
    disp(sprintf('%3.3f\t\t%0.5g\t\t%3.3f\t%0.5g\t%3.3f',out(imin(i),:))); 
end 

figure; hold on;
% note: even if the structure is perfectly symmetrical, due to the
% descretisation there may be slight numerical differences from span to
% span resulting in only 1 true max or minimum when in fact, theoretically,
% there are more.
[imax j] = find(output(2:nrows,2:ncols) == maxMt);
plot(output(2:nrows,1),-output(2:nrows,j+1),'r');
[imax j] = find(output(2:nrows,2:ncols) == minMt);
plot(output(2:nrows,1),-output(2:nrows,j+1),'k');
plot([0 max(x)],[0 0],'k');
title('BMDs for Max & Min cases');
ylabel('BM (kNm)');
xlabel('Distance along beam (m)');

figure; hold on;
plot(out(:,1),-out(:,2),'r');
plot(out(:,1),-out(:,4),'k');
plot([0 max(x)],[0 0],'k');
title('Maximum and Minimum BM Anywhere');
ylabel('BM (kNm)');
xlabel('Position of front axle (m)');

%figure;
%surface(-output(2:nrows,2:ncols));

%section envelopes
env(:,1) = max(output(2:nrows,2:ncols)');
env(:,2) = min(output(2:nrows,2:ncols)');
figure; hold on;
plot(x,-env(:,1),'r');
plot(x,-env(:,2),'k');
plot([0 max(x)],[0 0],'k');
title('Bending Moment Envelope');
ylabel('BM (kNm)');
xlabel('Distance along beam (m)');

if nargin > 2
    poi = varargin{1};
    % Point of Interest output
    % find the closest result to the POI
    i = 1;
    while x(i) < poi
        i = i+1;
    end
    maxMt = max(output(i+1,2:ncols)); [imax jmax] = find(output(i+1,2:ncols) == maxMt);
    minMt = min(output(i+1,2:ncols)); [imin jmin] = find(output(i+1,2:ncols) == minMt);
    maxPos = output(1,jmax+1); minPos = output(1,jmin+1);
    disp('Point of Interest results:');
    disp('Point    Max Mt  Load At     Min Mt      Load At');
    disp(sprintf('%3.3f \t %0.5g \t\t %3.3f \t\t %0.5g \t %3.3f',poi,maxMt,maxPos,minMt,minPos));
end
     
